<?php

use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\Form\Extension\Core\Type\FileType;
use Symfony\Component\Validator\Constraints as Assert;

class DntradeApiActionModuleFrontController extends ModuleFrontController {

    public function postProcess() {
	header('Content-Type: application/json');

	if (!$this->check_token_permission()) {
	    $this->ajaxRender(
		    json_encode([
		'status' => 0,
		'message' => "Не авторизовано"
	    ]));
	    exit;
	}
	$params = Tools::getValue('params');
	// Here we call all methods dynamically given by the path

	if (method_exists($this, Tools::getValue('action') . 'Action')) {
	    call_user_func([$this, Tools::getValue('action') . 'Action'], $params);
	    exit;
	}
    }

    public function checkAction() {
	return
		$this->ajaxRender(
			json_encode([
		    'status' => 1,
			])
	);
    }

    public function getstatusesAction() {
	// Отримання всіх статусів замовлень
	$order_statuses = OrderState::getOrderStates((int) $this->context->language->id);

	// Форматування отриманих статусів
	$formatted_statuses = array();
	foreach ($order_statuses as $status) {
	    $formatted_statuses[] = array(
		'id' => $status['id_order_state'],
		'title' => $status['name'],
	    );
	}

	// Повернення статусів у форматі JSON
	return $this->ajaxRender(json_encode(array('status' => 1, 'statuses' => $formatted_statuses)));
    }

    public function categoriesAction() {
	$categories = array();

	// Отримання всіх категорій товарів
	$categories_raw = Category::getCategories($this->context->language->id, true, false);

	// Перебір кожної категорії
	foreach ($categories_raw as $category) {
	    // Отримання URL зображення категорії (якщо воно є)
	    $categoryImageURL = '';

	    // Перевірка наявності зображення категорії
	    if (isset($category['image']) && $category['image']) {
		$categoryImageURL = $this->context->link->getMediaLink(_PS_CAT_IMG_DIR_ . $category['image']);
	    }

	    // Додавання категорії у форматі ['id' => ..., 'title' => ..., 'image' => ..., 'parent_id' => ...]
	    $categories[] = array(
		'id' => $category['id_category'],
		'title' => $category['name'],
		'image' => $categoryImageURL,
		'parent_id' => $category['id_parent'],
	    );
	}

	// Повернення категорій у форматі JSON через ajaxRender
	return $this->ajaxRender(json_encode(array('status' => 1, 'data' => $categories)));
    }

    public function productAction() {
	$productId = (int) Tools::getValue('id');

	// Отримуємо інформацію про товар за його ідентифікатором
	$product = new Product($productId, false, $this->context->language->id, $this->context->shop->id);

	if (!Validate::isLoadedObject($product)) {
	    return $this->ajaxRender(json_encode(array('error' => 'Товар не знайдено')));
	}

	// Викликаємо функцію, що повертає інформацію про товар
	$response = $this->getProductItem($product);
	return $this->ajaxRender(json_encode(array('status' => 1, 'product' => $response)));
    }

    public function productsAction() {
	// Отримання параметрів limit та offset з запиту GET
	$limit = (int) Tools::getValue('limit');
	$offset = (int) Tools::getValue('offset');

	// Отримання товарів з урахуванням limit та offset
	$products = Product::getProducts($this->context->language->id, $offset, $limit, 'id_product', 'DESC', false, true);

	// Побудова масиву з інформацією про товари
	$response = array();
	foreach ($products as $product) {
	    // Отримання інформації про кожен товар
	    $response[] = $this->getProductItemFromArray($product);
	}

	// Повернення товарів у форматі JSON
	return $this->ajaxRender(json_encode(['status' => 1, 'data' => $response]));
    }

    public function ordersAction() {
	// Отримання параметрів з POST-запиту
	$from_date = Tools::getValue('from_date');
	$limit = (int) Tools::getValue('limit');
	$offset = (int) Tools::getValue('offset');

// Отримання замовлень з використанням параметрів limit та offset
	$orders = $this->get_orders_with_filter($from_date, $limit ?: 100, $offset);

	// Побудова масиву з інформацією про замовлення
	$order_data = array();
	foreach ($orders as $order) {
	    $order_info = array(
		'order_id' => $order['id_order'],
		'customer_id' => $order['id_customer'],
		'customer_name' => $order['firstname'] . ' ' . $order['lastname'],
		'email' => $order['email'],
		'phone' => $order['phone'] ?? "",
		'payment_method' => $order['payment'],
		'address' => $order['city'] . ', ' . $order['address1'] . ($order['address2'] ? ', ' . $order['address2'] : ''),
		'shipping_method' => $order['carrier_name'] ?? "",
		'comment' => $order['note'],
		'total' => $order['total_paid'],
		'order_status' => $order['current_state'],
		'date_added' => $order['date_add'],
		'date_modified' => $order['date_upd']
	    );

	    // Отримання товарів у замовленні
	    $order_products = array();
	    $products = OrderDetail::getList($order['id_order']);
	    foreach ($products as $product) {
		$product_info = new Product($product['product_id'], false, $this->context->language->id, $this->context->shop->id);
		$order_products[] = array(
		    'id' => $product['product_id'],
		    'product' => $this->getProductItem($product_info), // Потрібно визначити, як отримати деталі продукту в PrestaShop
		    'quantity' => $product['product_quantity'],
		    'price' => $product['product_price'],
		    'total_price' => $product['total_price_tax_incl'],
		    'tax' => ""
		);
	    }

	    $order_info['products'] = $order_products;
	    $order_data[] = $order_info;
	}

	// Повернення інформації про замовлення у форматі JSON
	return $this->ajaxRender(json_encode(array('status' => 1, 'orders' => $order_data)));
    }

    public function uploadAction() {
	// Перевірка наявності параметру products у POST-запиті
	$products_json = Tools::getValue('products');
	if (!$products_json) {
	    return $this->ajaxRender(json_encode(array('status' => 0, 'message' => 'Параметр products обов\'язковий')));
	}

	// Декодування JSON-рядка у масив об'єктів
	$products = json_decode($products_json);

	// Масив для зберігання нових ідентифікаторів товарів
	$new_ids = array();

	// Перебір кожного товару
	foreach ($products as $item) {
	    // Побудова даних для оновлення або створення товару
	    $data = array(
		'name' => isset($item->title) ? $item->title : null,
		'price' => isset($item->price) ? $item->price : null,
		'price_old' => isset($item->price_old) ? $item->price_old : null,
		'quantity' => isset($item->quantity) ? $item->quantity : null,
		'description' => isset($item->description) ? $item->description : null,
		'reference' => isset($item->sku) ? $item->sku : null,
		'active' => isset($item->status) ? $item->status : null,
		'images' => isset($item->images) ? $item->images : null,
		    //'category_id' => isset($item->parent_id) ? $item->parent_id : null,
		    // Додавання інших даних, які можуть бути в товарі
	    );

	    // Додавання категорії до товару (якщо вона вказана)
	    if (isset($item->parent_id) && !empty($item->parent_id)) {
		// Отримання категорії за її ID
		$category = new Category($item->parent_id);

		// Перевірка, чи знайдено категорію
		if ($category->id) {
		    $data['id_category_default'] = $category->id; // Додавання категорії до даних товару
		}
	    }

	    // Оновлення або створення товару	
	    if (!$item->is_new) {
		// Оновлення існуючого товару
		$this->dntrade_api_update_product($item->id, $data);
	    } else {
		//return $this->ajaxRender(json_encode(array('status' => 1, 'data' => $item)));
		// Створення нового товару
		$product_id = $this->dntrade_api_create_product($data);
		$new_ids[$item->system_id] = $product_id;
	    }
	}

	// Повернення результату операції
	return $this->ajaxRender(json_encode(array('status' => 1, 'new' => $new_ids)));
    }

    function setstatusAction() {
	$order_id = Tools::getValue('order_id');
	if (empty($order_id)) {
	    return $this->ajaxRender(json_encode(array('status' => 0, 'message' => "Параметр order_id обов'язковий"), 400));
	}

	$status = Tools::getValue('status');
	if (empty($status)) {
	    return $this->ajaxRender(json_encode(array('status' => 0, 'message' => "Параметр status обов'язковий"), 400));
	}

	// Отримання замовлення за його ID
	$order = new Order((int) $order_id);
	if (!$order->id) {
	    return $this->ajaxRender(json_encode(array('status' => 0, 'message' => 'Замовлення не знайдено'), 404));
	}

	// Оновлення статусу замовлення
	$order->setCurrentState((int) $status);

	// Повернення результату оновлення
	return $this->ajaxRender(json_encode(array('status' => 1), 200));
    }

    private function get_orders_with_filter($from_date = null, $limit = 100, $offset = 0) {
	$context = Context::getContext();
	// Формуємо умови фільтрації
	$conditions = array();
	if ($from_date) {
	    $from_date = date("Y-m-d H:i:s", strtotime($from_date));
	    $conditions[] = 'o.`date_add` > \'' . pSQL($from_date) . '\' OR o.`date_upd` > \'' . pSQL($from_date) . '\'';
	}

	// Формуємо запит
	$sql = 'SELECT o.*, c.*, a.*, osl.`name` AS `order_state`, os.`color`, os.`logable`
            FROM `' . _DB_PREFIX_ . 'orders` o
            ' . Shop::addSqlAssociation('orders', 'o') . '
            LEFT JOIN `' . _DB_PREFIX_ . 'customer` c ON (c.`id_customer` = o.`id_customer`)
            LEFT JOIN `' . _DB_PREFIX_ . 'address` a ON (a.`id_address` = o.`id_address_delivery`)
            LEFT JOIN `' . _DB_PREFIX_ . 'order_state` os ON (os.`id_order_state` = o.`current_state`)
            LEFT JOIN `' . _DB_PREFIX_ . 'order_state_lang` osl ON (os.`id_order_state` = osl.`id_order_state` AND osl.`id_lang` = ' . (int) $context->language->id . ')
            WHERE ' . (!empty($conditions) ? implode(' AND ', $conditions) . ' AND ' : '') . ' 1
            ORDER BY o.`date_add` DESC
            LIMIT ' . (int) $offset . ', ' . (int) $limit;

	$orders = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($sql);

	return $orders;
    }

    protected function getProductItemFromArray($product) {
	// Отримання першого зображення товару
	$productImages = Image::getImages($this->context->language->id, $product['id_product']);
	$imageUrl = '';
	if (!empty($productImages)) {
	    $firstImage = array_shift($productImages);
	    $imageUrl = $this->context->link->getImageLink($product['link_rewrite'], (string) $firstImage['id_image'], 'home_default');
	}
	$categories = $product['id_category_default'];
	$categoryId = !empty($categories) ? (int) $categories[0] : null;

	$response = array(
	    'id' => $product['id_product'],
	    'title' => $product['name'],
	    'code' => $product['reference'],
	    'sku' => $product['reference'],
	    'quantity' => $product['quantity'],
	    'status' => (int) $product['active'],
		'barcode' => (int) $product['ean13'],
	    'link' => $this->context->link->getProductLink($product['id_product']),
	    'unit_title' => 'шт',
	    'image' => $imageUrl,
	    'parent' => !empty($categoryId) ? array(
	'id' => $categoryId,
	'title' => $this->getCategoryName($categoryId),
	'parent_id' => (int) $this->getParentCategoryId($categoryId)
	    ) : null,
	    'price' => $product['price'],
	    'description' => $product['description_short']
	);
	return $response;
    }

    protected function getProductItem($product) {
	// Отримання першого зображення товару
	$productImages = Image::getImages($this->context->language->id, $product->id);
	$imageUrl = '';
	if (!empty($productImages)) {
	    $firstImage = array_shift($productImages);
	    $imageUrl = $this->context->link->getImageLink($product->link_rewrite, (string) $firstImage['id_image'], 'home_default');
	}

	// Отримання категорій товару
	$categories = $product->getCategories();
	$categoryId = !empty($categories) ? (int) $categories[0] : null;

	// Побудова масиву з інформацією про товар
	$response = array(
	    'id' => $product->id,
	    'title' => $product->name,
	    'code' => $product->reference,
	    'sku' => $product->reference,
	    'quantity' => $product->quantity,
	    'status' => (int) $product->active,
	    'link' => $this->context->link->getProductLink($product->id),
	    'unit_title' => 'шт',
	    'image' => $imageUrl,
	    'parent' => !empty($categoryId) ? array(
	'id' => $categoryId,
	'title' => $this->getCategoryName($categoryId),
	'parent_id' => (int) $this->getParentCategoryId($categoryId)
	    ) : null,
	    'price' => $product->price,
	    'description' => $product->description
	);

	return $response;
    }

    protected function getCategoryName($categoryId) {
	$category = new Category($categoryId, $this->context->language->id);
	return $category->name;
    }

    protected function getParentCategoryId($categoryId) {
	$category = new Category($categoryId, $this->context->language->id);
	return (int) $category->id_parent;
    }

    protected function dntrade_api_upload_image_from_url($image_url, $product_id) {
	// Отримання інформації про зображення
	$image_data = file_get_contents($image_url);
	$filename = basename($image_url);

	// Отримання директорії зображень продуктів в PrestaShop
	$upload_dir = _PS_UPLOAD_DIR_;

	if (!file_exists($upload_dir)) {
	    mkdir($upload_dir, 0777, true);
	}

	// Шлях до файлу зображення в теку завантаження
	$file_path = $upload_dir . $filename;
	// Збереження зображення
	if (!file_put_contents($file_path, $image_data)) {
	    return null; // Помилка збереження зображення
	}

	// Додавання зображення до медіа-бібліотеки
	$image = new Image();
	$image->id_product = $product_id;
	$image->position = Image::getHighestPosition($product_id) + 1;
	if (!Image::getCover($image->id_product)) {
	    $image->cover = 1;
	} else {
	    $image->cover = 0;
	}
	$image->add();

	$new_path = $image->getPathForCreation();

	$imagesTypes = ImageType::getImagesTypes('products');
	$generate_hight_dpi_images = (bool) Configuration::get('PS_HIGHT_DPI');
	// Отримання реального формату зображення
	$imageInfo = getimagesize($file_path);

// Перевірка чи файл є JPEG зображенням
	if ($imageInfo['mime'] !== 'image/jpeg') {
	    $sourceImage = imagecreatefromstring(file_get_contents($file_path));
	    imagejpeg($sourceImage, $file_path);
	    imagedestroy($sourceImage);
	}

	foreach ($imagesTypes as $imageType) {
	    ImageManager::resize($file_path, $new_path . '-' . stripslashes($imageType['name']) . '.' . $image->image_format, $imageType['width'], $imageType['height'], $image->image_format);

	    if ($generate_hight_dpi_images) {
		ImageManager::resize($file_path, $new_path . '-' . stripslashes($imageType['name']) . '2x.' . $image->image_format, (int) $imageType['width'] * 2, (int) $imageType['height'] * 2, $image->image_format);
	    }
	}

	unlink($file_path);
	Hook::exec('actionWatermark', ['id_image' => $image->id, 'id_product' => $product_id]);
	$image->update();

	$shops = Shop::getContextListShopID();
	$image->associateTo($shops);
	$json_shops = [];

	foreach ($shops as $id_shop) {
	    $json_shops[$id_shop] = true;
	}
	//var_dump(_PS_TMP_IMG_DIR_ . 'product_mini_' . (int) $product_id . '_' . $this->context->shop->id . '.jpg');
	//@unlink(_PS_TMP_IMG_DIR_ . 'product_' . (int) $product_id . '.jpg');
	//@unlink(_PS_TMP_IMG_DIR_ . 'product_mini_' . (int) $product_id . '_' . $this->context->shop->id . '.jpg');
	// Отримання ID зображення
	$image_id = $image->id;

	return $image_id;
    }

    protected function dntrade_api_create_product($data) {
	$product = new Product();
	$product->name = $data['name'];
	$product->description = $data['description'];
	// Інші дані товару
	// Додавання категорії до товару
	if (!empty($data['id_category_default'])) {
	    $product->id_category_default = $data['id_category_default']; // Припустимо, що перший елемент масиву - це ID категорії за замовчуванням
	}

	// Додаємо зображення до товару
	if (!empty($data['image_id'])) {
	    $product->id_image = $data['image_id'];
	}

	// Збереження товару
	$product->save();
	$this->dntrade_api_update_product($product->id, $data);
	// Повернення ID створеного товару
	return $product->id;
    }

    protected function dntrade_api_update_product($product_id, $data) {
	$product = new Product($product_id);

	// Оновлення назви товару
	if (!empty($data['name'])) {
	    $product->name = $data['name'];
	}

	// Оновлення артикулу
	if ($data['reference'] !== null) {
	    $product->reference = $data['reference'];
	}

	// Оновлення ціни
	if ($data['price_old'] !== null && $data['price'] !== null) {
	    if ($data['price_old'] > $data['price']) {
		$product->price = $data['price_old'];
		$product->specific_prices = array(
		    array(
			'reduction' => $data['price_old'] - $data['price'],
			'reduction_type' => 'amount',
			'from_quantity' => 1,
			'from' => '0000-00-00 00:00:00',
			'to' => '0000-00-00 00:00:00'
		    )
		);
	    } else {
		$product->price = $data['price'];
	    }
	}
	// Оновлення кількості товару та його статусу наявності
	if ($data['quantity'] !== null) {
	    $product->quantity = $data['quantity'];
	    StockAvailable::setQuantity($product_id, 0, $data['quantity']);
	    if ($data['quantity'] > 0) {
		$product->out_of_stock = 2; // Товар в наявності
	    } else {
		$product->out_of_stock = 1; // Товар відсутній
	    }
	}

	// Завантаження зображення товару (якщо воно вказано)
	if ($data['images'] !== null) {
	    // Отримання шляху до зображення
	    $image_url = $data['images'][0];

	    // Завантаження зображення
	    $image_id = $this->dntrade_api_upload_image_from_url($image_url, $product_id);

	    // Додавання зображення до даних товару
	    if ($image_id) {
		$data['id_image'] = $image_id;
	    }
	}

	// Оновлення категорій товару
	if (!empty($data['categories'])) {
	    $product->id_category_default = $data['categories'][0]; // Припустимо, що перший елемент масиву - це ID категорії за замовчуванням
	    $product->category = $data['categories'];
	}

	// Збереження змін
	$product->save();
    }

    protected function check_token_permission() {
	// Отримання токену з заголовка Authorization
	$token = $this->getApiKeyFromHeaders();

	// Отримання токену, збереженого в PrestaShop
	$saved_token = Configuration::get('DNTRADE_API_TOKEN');
	if (empty($saved_token)) {
	    return false; // Неправильний токен
	}
	// Перевірка токену
	if ($token === $saved_token) {
	    return true; // Токен вірний
	} else {
	    return false; // Неправильний токен
	}
    }

    protected function getApiKeyFromHeaders() {
	// Check if getallheaders() function exists	
	if (function_exists('getallheaders')) {
	    $headers = getallheaders();
	    if (isset($headers['Apikey'])) {
			return $headers['Apikey'];
	    }
		if (isset($headers['apikey'])) {
			return $headers['apikey'];
	    }
		if (isset($headers['ApiKey'])) {
			return $headers['ApiKey'];
	    }
	} else {
	    // If getallheaders() function is not available
	    if (isset($_SERVER['HTTP_APIKEY'])) {
			return $_SERVER['HTTP_APIKEY'];
	    }
	}

	return null; // Return null if API key is not found
    }
}
